from subprocess import CalledProcessError
import logging

from ..context import Context

logger = logging.getLogger(__name__)

def __call__ (config):
    context = Context(config, logger)

    root_partition = find_root_partition(context)
    if not (root_partition and root_partition["options"].startswith("ro,")):
        return

    logger.info("Remounting root partition as writable")
    context.call_chroot(["mount", "-o", "remount,rw", root_partition["source"], "/"])

def find_root_partition (context):
    try:
        return [parse_line(line) for line in context.check_output_chroot(["findmnt", "--target", "/"]).split("\n")[1:] if line][0]
    except CalledProcessError:
        logger.info("Failed to find mount points.")
        return None
    except IndexError:
        return None

def parse_line (line):
    while "  " in line:
        line = line.replace("  ", " ")
    (target, source, fstype, options) = line.split(" ")
    return {
        "target": target,
        "source": source,
        "fstype": fstype,
        "options": options
    }
