import logging
import os

from subprocess import PIPE

from ..capturecast import CaptureCast
from ..context import Context
from ..packages import getSystemPackages
from ..registration import get_specified_repos
from ..commands import install

logger = logging.getLogger(__name__)

def __call__ (config):
    context = Context(config, logger)
    capturecast = CaptureCast(context)
    
    logger.info("Updating Cattura Platform")
    
    context.apt_update()
    
    logger.info("Fetching list of installed packages")
    installed_packages = [package for package in getSystemPackages(context) if "installed" in package.status]
    logger.info("{} packages found.".format(len(installed_packages)))
    
    cattura_repos = get_specified_repos(context, config.update_from)    
    if not len(cattura_repos):
        raise Exception("No valid repositories specified")
    
    cattura_packages = []
    for repo in [repo for repos in cattura_repos.itervalues() for repo in repos]:
        logger.info("Fetching package list from [{}]".format(repo.url))
        cattura_packages += repo.packages()
    
    logger.info("{} packages found.".format(len(cattura_packages)))
    
    logger.info("Finding packages that need to be updated")
    packages_to_update = [package for package in installed_packages if len([cattura_package for cattura_package in cattura_packages if cattura_package.package == package.package]) > 0]
    logger.info("{} packages found.".format(len(packages_to_update)))
    
    updatePackages(context, packages_to_update)
    if not config.norestart and capturecast.is_running() and context.ask_yesno("Updates installed. Restart CaptureCast?"):
        capturecast.restart()

def updatePackages (context, packages):
    process_env = os.environ.copy()
    process_env["CATTURA_NO_RESTART"] = "y"
    context.call_chroot(install([package.package for package in packages]), env=process_env, stdin=PIPE)